import _extends from "@babel/runtime/helpers/esm/extends";
import React, { useCallback } from 'react';
import PropTypes from 'prop-types';
import { field, createComponentMemo } from '../../prop-types';
import MoleculeSelectField from '@s-ui/react-molecule-select-field';
import MoleculeSelectOption from '@s-ui/react-molecule-dropdown-option';
import IconChevronDown from '../../Icons/IconChevronDown';

var DefaultSelect = function DefaultSelect(_ref) {
  var select = _ref.select,
      tabIndex = _ref.tabIndex,
      onChange = _ref.onChange,
      onBlur = _ref.onBlur,
      size = _ref.size,
      errors = _ref.errors,
      alerts = _ref.alerts,
      renderer = _ref.renderer;
  var errorMessages = errors[select.id];
  var alertMessages = alerts[select.id];
  var onChangeCallback = useCallback(function (evt, _ref2) {
    var value = _ref2.value;
    return onChange(select.id, value);
  }, [onChange, select]);

  var onBlurCallback = function onBlurCallback() {
    return onBlur(select.id);
  };

  var _select$datalist = select.datalist,
      datalist = _select$datalist === void 0 ? [] : _select$datalist; // transform constraints to props

  var constraints = select.constraints || [];
  var constraintsProps = {};
  constraintsProps = constraints.reduce(function (acc, constraint) {
    var _constraint$property;

    if ((constraint == null ? void 0 : (_constraint$property = constraint.property) == null ? void 0 : _constraint$property.notnull) === '') {
      return _extends({}, acc, {
        required: true
      });
    } else {
      return acc;
    }
  }, constraintsProps);

  var selectProps = _extends({
    id: select.id,
    label: select.label,
    name: select.name,
    placeholder: select.hint,
    iconArrowDown: React.createElement(IconChevronDown, null),
    value: select.value || '',
    onChange: onChangeCallback,
    onBlur: onBlurCallback,
    helpText: select.help,
    tabIndex: tabIndex
  }, select.disabled && {
    disabled: true
  }, {}, select.hidden && {
    hidden: true
  }, {}, !!errorMessages && {
    errorText: errorMessages.join('\n')
  }, {}, !!alertMessages && {
    alertText: alertMessages.join('\n')
  }, {
    selectSize: size
  }, constraintsProps);

  if (selectProps.hidden) {
    return null;
  }

  var rendererResponse = renderer({
    id: select.id,
    innerProps: _extends({}, selectProps, {
      datalist: datalist
    })
  }); // render custom component

  if (React.isValidElement(rendererResponse)) return rendererResponse; // render SUI component

  return React.createElement("div", {
    className: "sui-FormBuilder-field sui-FormBuilder-DefaultSelect sui-FormBuilder-" + selectProps.id
  }, selectProps.label && React.createElement("label", {
    className: "sui-FormBuilder-label",
    htmlFor: selectProps.id
  }, selectProps.label), React.createElement(MoleculeSelectField, Object.assign({}, selectProps, rendererResponse), datalist.map(function (data) {
    return React.createElement(MoleculeSelectOption, {
      key: data.value,
      value: data.value
    }, data.text);
  })));
};

DefaultSelect.displayName = 'DefaultSelect';
export default React.memo(DefaultSelect, createComponentMemo('select'));