import _regeneratorRuntime from "@babel/runtime/regenerator";
import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";

/* eslint-disable no-console */
import { operators } from './operators';
import { IN, NIN, EXISTS, NEXISTS, CHANGED, REMOTE, EQUALS, GREATERTHAN, LESSTHAN } from './constants';
import { changeFieldById, fieldsToQP } from './fields';

var fetch = function fetch(config) {
  return new Promise(function (resolve, reject) {
    var url = config.url,
        headers = config.headers;
    var request = new window.XMLHttpRequest();

    request.onreadystatechange = function () {
      if (request.readyState === window.XMLHttpRequest.DONE) {
        if (request.status === 200) {
          resolve(request.response);
        } else {
          reject(Error(request.status));
        }
      }
    };

    request.onerror = function () {
      reject(Error('Network Error'));
    };

    request.open('GET', url, true);
    headers && Object.entries(headers).map(function (_ref) {
      var header = _ref[0],
          value = _ref[1];
      return request.setRequestHeader(header, value);
    });
    request.responseType = 'json';
    request.withCredentials = true;
    request.send();
  });
};

export var shouldApplyRule = function shouldApplyRule(fields, changeField, locale) {
  return function (when) {
    // Reduce works as AND operator, all rules should be true
    var isValid = when.reduce(function (acc, rule) {
      // if (rule.id !== changeField) {
      if (!when.some(function (rule) {
        return rule.id === changeField;
      })) {
        return false;
      }

      var isValid;

      switch (rule.operator) {
        case IN:
          isValid = operators.IN(rule.id, rule.value, fields, changeField);
          break;

        case NIN:
          isValid = operators.NIN(rule.id, rule.value, fields, changeField);
          break;

        case EXISTS:
          isValid = operators.EXISTS(rule.id, fields);
          break;

        case NEXISTS:
          isValid = operators.NEXISTS(rule.id, fields);
          break;

        case CHANGED:
          isValid = operators.CHANGED(rule.id, changeField);
          break;

        case EQUALS:
          isValid = operators.EQUALS(rule.id, rule.value, fields, locale);
          break;

        case GREATERTHAN:
          isValid = operators.GREATERTHAN(rule.id, rule.value, fields, locale);
          break;

        case LESSTHAN:
          isValid = operators.LESSTHAN(rule.id, rule.value, fields, locale);
          break;

        default:
          console.warn("Unkown operator " + rule.operator);
          isValid = false;
      }

      return acc && isValid;
    }, true);
    return isValid;
  };
};
export var fetchRemoteFields = function fetchRemoteFields(fields, formID, responseInterceptor, requestInterceptor) {
  return (
    /*#__PURE__*/
    function () {
      var _ref2 = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee3(fieldsToChanges) {
        var remoteFieldsToChange;
        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return Promise.all(Object.entries(fieldsToChanges).filter(function (_ref3) {
                  var field = _ref3[0],
                      nextValue = _ref3[1];
                  return nextValue.remote === REMOTE;
                }).map(
                /*#__PURE__*/
                function () {
                  var _ref4 = _asyncToGenerator(
                  /*#__PURE__*/
                  _regeneratorRuntime.mark(function _callee2(_ref5) {
                    var field, nextValue, defaultUrl, defaultConfig, requestInterceptorConfig, config, remote, resetValue;
                    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            field = _ref5[0], nextValue = _ref5[1];
                            defaultUrl = "https://ptaformbuilder-classifiedads.spain.advgo.net/fieldrules/" + field + "?" + fieldsToQP(fields, formID);
                            defaultConfig = {
                              url: defaultUrl
                            };
                            _context2.next = 5;
                            return requestInterceptor({
                              fieldID: field,
                              fields: fields
                            });

                          case 5:
                            requestInterceptorConfig = _context2.sent;
                            // Config must be follow the Axios pattern
                            // https://github.com/axios/axios
                            config = requestInterceptorConfig || defaultConfig;
                            remote = nextValue.remote, resetValue = _objectWithoutPropertiesLoose(nextValue, ["remote"]);
                            return _context2.abrupt("return", fetch(config).then(
                            /*#__PURE__*/
                            function () {
                              var _ref6 = _asyncToGenerator(
                              /*#__PURE__*/
                              _regeneratorRuntime.mark(function _callee(json) {
                                var url, nextJSON;
                                return _regeneratorRuntime.wrap(function _callee$(_context) {
                                  while (1) {
                                    switch (_context.prev = _context.next) {
                                      case 0:
                                        url = config.url;
                                        _context.next = 3;
                                        return responseInterceptor({
                                          field: field,
                                          url: url,
                                          response: typeof json === 'string' || json instanceof String ? JSON.parse(json) : json
                                        });

                                      case 3:
                                        nextJSON = _context.sent;
                                        return _context.abrupt("return", [field, _extends({}, resetValue, {}, nextJSON)]);

                                      case 5:
                                      case "end":
                                        return _context.stop();
                                    }
                                  }
                                }, _callee);
                              }));

                              return function (_x3) {
                                return _ref6.apply(this, arguments);
                              };
                            }())["catch"](function (error) {
                              console.error("FAILED requesting remote nextValue for " + field + " with error: " + error);
                              return [field, {}];
                            }));

                          case 9:
                          case "end":
                            return _context2.stop();
                        }
                      }
                    }, _callee2);
                  }));

                  return function (_x2) {
                    return _ref4.apply(this, arguments);
                  };
                }()));

              case 2:
                remoteFieldsToChange = _context3.sent;
                return _context3.abrupt("return", remoteFieldsToChange);

              case 4:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3);
      }));

      return function (_x) {
        return _ref2.apply(this, arguments);
      };
    }()
  );
};
export var applyRules =
/*#__PURE__*/
function () {
  var _ref7 = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee4(fields, rules, changeField, formID, responseInterceptor, requestInterceptor, locale) {
    var shouldApplyRuleForFieldsAndChangeField, fetchRemoteFieldsForFieldsAndFormID, fieldsToChanges, remoteFieldsToChange, nextFields;
    return _regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            shouldApplyRuleForFieldsAndChangeField = shouldApplyRule(fields, changeField, locale);
            fetchRemoteFieldsForFieldsAndFormID = fetchRemoteFields(fields, formID, responseInterceptor, requestInterceptor);
            fieldsToChanges = Object.entries(rules).reduce(function (acc, _ref8) {
              var _ref9;

              var idField = _ref8[0],
                  rulesField = _ref8[1];
              // Find works as OR operator, and we want the first match
              var firstMatch = rulesField.find(function (ruleField) {
                return shouldApplyRuleForFieldsAndChangeField(ruleField.when);
              });
              return _extends({}, acc, {}, firstMatch && (_ref9 = {}, _ref9[idField] = _extends({}, firstMatch.then.data, {}, firstMatch.then.remote && {
                remote: REMOTE
              }), _ref9));
            }, {});
            _context4.next = 5;
            return fetchRemoteFieldsForFieldsAndFormID(fieldsToChanges);

          case 5:
            remoteFieldsToChange = _context4.sent;
            nextFields = Object.entries(fieldsToChanges).filter(function (_ref10) {
              var idField = _ref10[0],
                  nextValue = _ref10[1];
              return nextValue !== REMOTE;
            }).reduce(function (acc, _ref11) {
              var idField = _ref11[0],
                  nextValue = _ref11[1];
              return changeFieldById(acc, idField, nextValue);
            }, fields);
            nextFields = remoteFieldsToChange.reduce(function (acc, _ref12) {
              var idField = _ref12[0],
                  nextValue = _ref12[1];
              return changeFieldById(acc, idField, nextValue);
            }, nextFields);
            return _context4.abrupt("return", nextFields);

          case 9:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));

  return function applyRules(_x4, _x5, _x6, _x7, _x8, _x9, _x10) {
    return _ref7.apply(this, arguments);
  };
}();